<?php
class VLCServer {
	/**
	* @access private
	* @var int
	*/
	private $lVolume;

	/**
	* @access private
	* @var int
	*/
	private $lScale;

	/**
	* @access private
	* @var string
	*/
	private $lVLCExecutable;

	/**
	* @access private
	* @var int
	*/
	private $lChannels;
	
	
	/**
	* @access private
	* @var array
	*/
	private $lProfiles;

	/**
	* Constructor. This creates a VLC Server object. Some defaults are already set. These are: volume, scale and audio channels. It alse detects if it is running on Windows or Linux.
	* @return VLCServer
	*/
	function __construct() {		
		$this->lProfiles = array();
		
		/*
			Here you can change some codec settings.
			Possible Audio codecs:
				1. mp4a
				2. mp3
				3. mpga
		*/
		
		//3G Profile (BETA)
		//$this->lProfiles[] = array("name" => "3G", "width" => 240, "height" => 160, "videocodec" => "h264", "videobitrate" => 150, "videofps" => 15, "audiocodec" => "mp4a", "audiobitrate" => 32, "audiosamplerate" => 44100);
		// Mobile Profile
		$this->lProfiles[] = array("name" => "Mobile", "width" => 416, "height" => 234, "videocodec" => "h264", "videobitrate" => 200, "videofps" => 24, "audiocodec" => "mp4a", "audiobitrate" => 92, "audiosamplerate" => 48000);
		// Main Profile
		$this->lProfiles[] = array("name" => "SD", "width" => 720, "height" => 404, "videocodec" => "h264", "videobitrate" => 512, "videofps" => 24, "audiocodec" => "mp4a", "audiobitrate" => 128, "audiosamplerate" => 48000);
		// HD Profile
		//$this->lProfiles[] = array("name" => "HD", "width" => 1280, "height" => 720, "videocodec" => "h264", "videobitrate" => 2048, "videofps" => 25, "audiocodec" => "mp4a", "audiobitrate" => 192, "audiosamplerate" => 48000);
		
		$this->lVolume = 80;
		$this->lScale = 1;
		$this->lChannels = 2;
		
		$this->lVLCExecutable = Settings::getVLCExecutable();	
	}

	/**
	* Start the VLC Server by executing the vlc command line. You have to set all options before starting this server. If the server is started, you should be able to connect to it and watch some tv.
	* @param boolean $pDebug
	* @return string
	*/
	public function startServer($pStreamURL,$pChannelObj) {
		global $gDebugObj;
		$lCurrentStatus = $this->getCurrentStatus();
		if ($lCurrentStatus["streamurl"] != $pStreamURL) {
			$this->stopServer();
			sleep(2);
			if (Settings::getGgrabLocation() != "") {
				$lCMD = Settings::getGgrabLocation() . " -host " . Settings::getDreamboxIP() . " -q -o - -p 0x1006 0x1106 | " . $this->lVLCExecutable . " - ";
			} else {
				$lCMD = $this->lVLCExecutable . " \"" . $pStreamURL . "\" vlc://quit ";
				if (Settings::getVLCAudioLanguage() != '') {
					$lAudioTracks = explode(',',Settings::getVLCAudioLanguage());
					if (is_numeric($lAudioTracks[0])) {
						$lCMD .= ' --audio-track=' . $lAudioTracks[0];
					} else {
						$lCMD .= ' --audio-language=' . Settings::getVLCAudioLanguage();
					}
				}
				if (Settings::getVLCSubtitleLanguage() != '') {
					$lSubTracks = explode(',',Settings::getVLCSubtitleLanguage());
					if (is_numeric($lSubTracks[0])) {
						$lCMD .= ' --sub-track=' . $lSubTracks[0];
					} else {
						$lCMD .= ' --sub-language=' . Settings::getVLCSubtitleLanguage();
					}
				}
			}
			$lPortNumbers = 20000;
			
			$lTransCode = "#duplicate{";
			foreach ($this->lProfiles as $lProfile) {
				$lCodecParams = "";
				switch ($lProfile["videocodec"]) {
					case "h264":
						$lCodecParams = "venc=x264{aud,keyint=" . ($lProfile["videofps"]*2) . ",profile=baseline,level=31,bframes=0,ref=1,nocabac,qpmax=51,qpmin=10,me=hex,subq=6,subme=7,qcomp=0.6,fps=" . ($lProfile["videofps"]/2) . "},";
						//$lCodecParams = "venc=x264{aud,keyint=30,profile=baseline,level=30,ref=1},";
					break;
				}
				$lTransCode .= "dst={transcode{threads=2,deinterlace,vcodec=". $lProfile["videocodec"] . "," . $lCodecParams . "vb=". $lProfile["videobitrate"] . ",scale=". $this->lScale .",acodec=". $lProfile["audiocodec"] .",ab=" . $lProfile["audiobitrate"] . ",channels=". $this->lChannels .",samplerate=" . $lProfile["audiosamplerate"] . ",width=". $lProfile["width"] .",height=". $lProfile["height"] . ",audio-sync,fps=" . ($lProfile["videofps"]/2) . "}";
				$lTransCode .= ":duplicate{";
			
				if (Settings::isWowzaEnabled()) {
					// Send stream through Wowza.
					$lTransCode .= "dst=rtp{dst=". Settings::getVLCIP() .",port=" . $lPortNumbers . ",mux=ts,ttl=20,sdp=http://". Settings::getVLCIP() .":". Settings::getVLCPort() ."/" . $this->vlcStreamUrlFormat($pChannelObj->getName() . "_" . $lProfile["name"]) . ".sdp},";
				} else if (Settings::isVLCiPhoneEnabled()) {
					// iPhone chunks (For iOs devices and Fash players on desktop browsers)
					$lTransCode .= "dst=gather:std{access=livehttp{seglen=10,delsegs=true,numsegs=3,index=" . getcwd() . "/iphone/stream/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_" . $lProfile["name"] . ".m3u8,index-url=" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_" . $lProfile["name"] . "-########.ts},mux=" . ($lProfile["audiocodec"] == 'mp4a' ? 'ts' : 'raw') . "{use-key-frames},dst=" . getcwd() . "/iphone/stream/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_" . $lProfile["name"] . "-########.ts},";
				} else {
					// Old way of flash streaming. For VLC players below version 2.x
					$lTransCode .= "dst=gather:std{access=http,mime=\"video/mp4\",dst=" . Settings::getVLCPlayerIP(true) .":". Settings::getVLCPort() . "/" . $this->vlcStreamUrlFormat($pChannelObj->getName() . "_" . $lProfile["name"]) . ".mp4,mux=ffmpeg{mux=flv}},";
				}
				
				if (!Settings::isWowzaEnabled() && $lProfile["name"] == 'Mobile') {
					// RTSP Mobile server
					$lTransCode .= "dst=rtp{dst=". Settings::getVLCPlayerIP(true) .",ttl=20,sdp=rtsp://:". Settings::getVLCPort() . "/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_" . $lProfile["name"] . ".sdp},";
				}
				
				$lTransCode = substr(trim($lTransCode),0,-1) . "}},";				
				$lPortNumbers += 10;
			}
			$lTransCode = substr(trim($lTransCode),0,-1) . "}";
			
			if (Utils::isLinux() || Utils::isMacOSX()) $lTransCode = "\"" . $lTransCode . "\"";
			$lCMD .= " --sout=" . $lTransCode;
			Utils::execute($lCMD);
			if (Settings::isWowzaEnabled()) $this->saveSDPfiles($pChannelObj);
			else sleep(10); // Wait 10 seconds to get VLC up to speed and creating the first chunks
			if (Settings::getDebug()) $gDebugObj->setDebugMessage("VLC Command:\n".$lCMD);
		}
		if (Settings::isVLCiPhoneEnabled() || Settings::isFullWowzaServer()) $this->setIphoneStream($pChannelObj->getName());
		return $this->getPlayerUrl($pChannelObj);
	}
	
	public function getPlayerUrl($pChannelObj) {
		if (Utils::isiPhone()) {
			if (Settings::isWowzaEnabled() && Settings::isFullWowzaServer()) {
				if (Settings::isMultiBitRate()) {
					$lReturnValue = "http://" . Settings::getWowzaServerName() . ":1935/dreambox/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . ".smil/playlist.m3u8";
				} else {
					$lReturnValue = "http://" . Settings::getWowzaServerName() . ":1935/dreambox/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_3G.sdp/playlist.m3u8";
				}
			} else if (Settings::isVLCiPhoneEnabled()) {
				$lReturnValue = $this->vlcStreamUrlFormat($pChannelObj->getName(true));
			} else {
				$lReturnValue = '';
			}
		} else if (Utils::isMobileDevice()) {
			if (Settings::isWowzaEnabled()) {
				$lReturnValue = "rtsp://" . Settings::getWowzaServerName(true) . ":1935/dreambox/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_Mobile.sdp";
			} else {
				$lReturnValue = "rtsp://" . Settings::getVLCPlayerIP(true) . ":" . Settings::getVLCPort() . "/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_Mobile.sdp";
			}
		} else if (Settings::isWowzaEnabled()) {
			$lReturnValue = "{streamer:'rtmp://" . Settings::getWowzaServerName() . "/dreambox',levels: [";
			foreach ($this->lProfiles as $lProfile) {
				if ($lProfile["name"] == "3G") continue;
				$lReturnValue .= "{bitrate: " . ($lProfile["videobitrate"]+$lProfile["audiobitrate"]) . ", file: '" . $this->vlcStreamUrlFormat($pChannelObj->getName() . "_" . $lProfile["name"]) . ".sdp', width: " . $lProfile["width"] . " },";
			}
			$lReturnValue = substr($lReturnValue,0,-1) .  "],type:'rtmp'}";
		} else if (Settings::isVLCiPhoneEnabled()) {
			$lReturnValue = $this->vlcStreamUrlFormat($pChannelObj->getName(true));
		} else {
			$lReturnValue = "{levels: [";
			foreach ($this->lProfiles as $lProfile) {
				if ($lProfile["name"] == "3G") continue;
				$lReturnValue .= "{bitrate: " . ($lProfile["videobitrate"]+$lProfile["audiobitrate"]) . ", file: 'http://" . Settings::getVLCPlayerIP() . ":" . Settings::getVLCPort() . "/" . $this->vlcStreamUrlFormat($pChannelObj->getName() . "_" . $lProfile["name"]) . ".mp4', width: " . $lProfile["width"] . " },";
			}
			$lReturnValue = substr($lReturnValue,0,-1) .  "],type:'video'}";
		}
		return $lReturnValue;
	}

	public function getCurrentStatus() {
		$lReturnArray = array();
		$lReturnArray["channel"] = "";
		$lReturnArray["streamurl"] = "";
		$lVLCServer = 'http://' . Settings::getVLCIP() . ':' . Settings::getVLCPort();
		$lVLCServer = 'http://';
		
		$lVLCServer = 'vlc://quit';
		if (Utils::isLinux() || Utils::isMacOSX()) {
			$lData = trim(shell_exec("ps ax | grep vlc | grep -v grep"));
			$lStartPos = stripos($lData,$lVLCServer);
			$lData = explode(' ',trim(substr($lData,0,$lStartPos)));
			$lReturnArray["streamurl"] = end($lData);
		}
		if (substr($lReturnArray["streamurl"],0,7) == "http://") {
			$lChannelData = parse_url($lReturnArray["streamurl"]);
			$lReturnArray["channel"] = substr($lChannelData['query'],4,-9);
		} else {
			$lReturnArray["channel"] = $lReturnArray["streamurl"];
		}
		$lReturnArray["channel"] = str_replace("%20"," ",$lReturnArray["channel"]);
		return $lReturnArray;
	}

	/**
	* Stop the VLC Server. This is handy to spare some system resources.
	*/
	public function stopServer() {
		if (Utils::isLinux()) {
			$lCMD = "killall -I -q vlc ruby ffmpeg;rm " . getcwd() . "/apps/wowza/content/*;rm " . getcwd() . "/iphone/stream/*";
		} elseif(Utils::isMacOSX()) {
			$lCMD = "killall VLC;rm " . getcwd() . "/apps/wowza/content/*;rm " . getcwd() . "/iphone/stream/*";
		} elseif(Utils::isWindows()) {
			// But you need to download this one: http://www.beyondlogic.org/solutions/processutil/processutil.htm
			// And install it in a dir called apps in the restream root dir
			$lCMD = getcwd() . "\apps\Process.exe -k vlc.exe";
			Utils::execute($lCMD);
			$lCMD = "del " . getcwd() . "/apps/wowza/content/*;del " . getcwd() . "/iphone/stream/*";
		} else {
			$lCMD = "";
		}
		Utils::execute($lCMD);
	}

	private function saveSDPfiles($pChannelObj) {
		$lSmillContent = "<smil><head></head><body><switch>";
		foreach($this->lProfiles as $lProfile) {
			
			$lVLCLocation = "http://". Settings::getVLCIP() .":". Settings::getVLCPort();
			$lSDPFileName = $this->vlcStreamUrlFormat($pChannelObj->getName() . "_" . $lProfile["name"]) . ".sdp";
			$lSDPContent = "";
		
			for ($i = 0; $i < 15; $i++) {
				$lSDPContent = @file_get_contents($lVLCLocation ."/" . $lSDPFileName);
				if ($lSDPContent != "" && ((stripos($lSDPContent,"m=video") !== false ) || ($i > 12 && stripos($lSDPContent,"m=audio") !== false))) {
					file_put_contents(getcwd() . "/apps/wowza/content/" . $lSDPFileName,$lSDPContent);
					if ($lProfile["name"] != "3G") $lSmillContent .= "<video src=\"" . $lSDPFileName . "\" system-bitrate=\"" . (($lProfile["videobitrate"] + $lProfile["audiobitrate"]) * 1024). "\"/>";
					break;
				} 
				sleep(1);
			}
		}
		$lSmillContent .= "</switch></body></smil>";
		if (Settings::isMultiBitRate()) file_put_contents(getcwd() . "/apps/wowza/content/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . ".smil",$lSmillContent);
	}
	
	public function setIphoneStream($pChannelName) {
		$lsave = false;
		$lMultiBitrate = "#EXTM3U\n";
		foreach ($this->lProfiles as $lProfile) {
			for ($i = 0; $i < 5; $i++) {
				if (file_exists(getcwd() . "/iphone/stream/" . $this->vlcStreamUrlFormat($pChannelName) . "_" . $lProfile["name"] . ".m3u8")) {
					$lMultiBitrate .= "#EXT-X-STREAM-INF:PROGRAM-ID=1, BANDWIDTH=" . round(($lProfile["videobitrate"]+$lProfile["audiobitrate"]) * 1024) . "\n" . $this->vlcStreamUrlFormat($pChannelName) . "_" . $lProfile["name"] . ".m3u8\n";
					$lsave = true;
					break;
				}
				sleep(1);
			}			
		}
		if ($lsave) file_put_contents(getcwd() . "/iphone/stream/" . $this->vlcStreamUrlFormat($pChannelName) . ".m3u8",$lMultiBitrate);
	}

	private function vlcStreamUrlFormat($pUrl) {
		$pUrl = trim($pUrl);
		return str_replace(array(" ","'"),array("_",""),preg_replace("/[^a-zA-Z0-9_\s]/","",$pUrl));
	}

	/**
	* Check if the server is still running.
	* @return boolean
	*/
	public function isServerRunning() {
		if (Utils::isLinux() || Utils::isMacOSX()) {
			$lData = trim(shell_exec("ps ax | grep -i vlc | grep -v grep | grep " . Settings::getDreamboxIP()));
		} elseif(Utils::isWindows()) {
			// On windows it is not possible to distinquish different vlc processes... so if there is a VLC proces, the server is running...
			$lCMD = getcwd() . "\apps\Process.exe | " . getcwd()  . "\apps\grep.exe vlc";
			$lData = trim(shell_exec($lCMD));
		}
		return (strlen($lData) > 0);
	}
}
?>
